# Generated by Django 2.2.24 on 2021-09-09 15:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0009_auto_20210708_1639'),
    ]

    operations = [
        migrations.AddField(
            model_name='canvasenterprisecustomerconfiguration',
            name='idp_id',
            field=models.CharField(blank=True, help_text='If provided, will be used as IDP slug to locate remote id for learners', max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='idp_id',
            field=models.CharField(blank=True, help_text='If provided, will be used as IDP slug to locate remote id for learners', max_length=255, null=True),
        ),
    ]
