"""
Enterprise Integrated Channel SAP SuccessFactors Django application initialization.
"""

from django.apps import AppConfig


class SAPSuccessFactorsConfig(AppConfig):
    """
    Configuration for the Enterprise Integrated Channel SAP SuccessFactors Django application.
    """
    name = 'integrated_channels.sap_success_factors'
    verbose_name = "Enterprise SAP SuccessFactors Integration"
