# Generated by Django 3.2.15 on 2022-10-31 18:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('degreed', '0025_auto_20221031_1855'),
    ]

    operations = [
        migrations.RenameField(
            model_name='degreedlearnerdatatransmissionaudit',
            old_name='completed_timestamp',
            new_name='degreed_completed_timestamp',
        ),
        migrations.AddField(
            model_name='degreedlearnerdatatransmissionaudit',
            name='completed_timestamp',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
