# Generated by Django 3.2.16 on 2022-11-22 10:09

from django.db import migrations, models
import enterprise.validators


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0164_enterprisecatalogquery_include_exec_ed_2u_courses'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomerreportingconfiguration',
            name='pgp_encryption_key',
            field=models.TextField(blank=True, help_text='The key for encryption, if PGP encrypted file is required.', null=True, validators=[enterprise.validators.validate_pgp_key], verbose_name='PGP Encryption Key'),
        ),
    ]
