# Generated by Django 3.2.23 on 2024-01-17 12:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0030_merge_0028_auto_20231116_1826_0029_auto_20231106_1233'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='password',
        ),
        migrations.RemoveField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='token',
        ),
        migrations.RemoveField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='username',
        ),
        migrations.RemoveField(
            model_name='moodleenterprisecustomerconfiguration',
            name='password',
        ),
        migrations.RemoveField(
            model_name='moodleenterprisecustomerconfiguration',
            name='token',
        ),
        migrations.RemoveField(
            model_name='moodleenterprisecustomerconfiguration',
            name='username',
        ),
    ]
