# Generated by Django 2.2.19 on 2022-05-12 14:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0155_add_is_relinkable_to_enterprise_customer_user'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalsystemwideenterpriseuserroleassignment',
            name='is_active',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='systemwideenterpriseuserroleassignment',
            name='is_active',
            field=models.BooleanField(default=True),
        ),
    ]
